# Generated by Django 3.2 on 2021-04-30 14:33

import datetime

import colorfield.fields
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models

import wkz.models


class Migration(migrations.Migration):

    dependencies = [
        ("wkz", "0006_auto_20210321_1812"),
    ]

    operations = [
        migrations.AlterField(
            model_name="activity",
            name="date",
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name="Date"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="description",
            field=models.CharField(blank=True, max_length=600, null=True, verbose_name="Description"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="distance",
            field=models.FloatField(blank=True, default=0, null=True, verbose_name="Distance"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="duration",
            field=models.DurationField(default=datetime.timedelta(seconds=1800), verbose_name="Duration"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="evaluates_for_awards",
            field=models.BooleanField(default=True, verbose_name="Consider Activity for Awards"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="is_demo_activity",
            field=models.BooleanField(default=False, verbose_name="Is this a Demo Activity"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="name",
            field=models.CharField(default="unknown", max_length=200, verbose_name="Activity Name"),
        ),
        migrations.AlterField(
            model_name="activity",
            name="sport",
            field=models.ForeignKey(
                default=wkz.models.default_sport,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                to="wkz.sport",
                verbose_name="Sport",
            ),
        ),
        migrations.AlterField(
            model_name="bestsection",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="lap",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="lap",
            name="label",
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name="Label"),
        ),
        migrations.AlterField(
            model_name="settings",
            name="delete_files_after_import",
            field=models.BooleanField(default=False, verbose_name="Delete fit Files after Copying "),
        ),
        migrations.AlterField(
            model_name="settings",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="settings",
            name="path_to_garmin_device",
            field=models.CharField(default="/run/user/1000/gvfs/", max_length=120, verbose_name="Path to Garmin Device"),
        ),
        migrations.AlterField(
            model_name="settings",
            name="path_to_trace_dir",
            field=models.CharField(
                default="/workspaces/workoutizer/tracks", max_length=120, verbose_name="Path to Traces Directory"
            ),
        ),
        migrations.AlterField(
            model_name="sport",
            name="color",
            field=colorfield.fields.ColorField(default="#42FF71", max_length=18, verbose_name="Color"),
        ),
        migrations.AlterField(
            model_name="sport",
            name="evaluates_for_awards",
            field=models.BooleanField(default=True, verbose_name="Consider Sport for Awards"),
        ),
        migrations.AlterField(
            model_name="sport",
            name="icon",
            field=models.CharField(max_length=24, verbose_name="Icon"),
        ),
        migrations.AlterField(
            model_name="sport",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="sport",
            name="name",
            field=models.CharField(max_length=24, unique=True, verbose_name="Sport Name"),
        ),
        migrations.AlterField(
            model_name="traces",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]



from django import forms

from wkz.models import Activity, Settings, Sport

DATETIMEPICKER_FORMAT = "%m/%d/%Y %I:%M %p"


def set_field_attributes(visible_fields):
    for visible in visible_fields:
        if visible.name == "date":  # because it would overwrite the required 'datetimepicker' class
            continue
        else:
            visible.field.widget.attrs["class"] = "form-control"


class AddSportsForm(forms.ModelForm):
    class Meta:
        model = Sport
        exclude = ("created", "modified")

    def __init__(self, *args, **kwargs):
        super(AddSportsForm, self).__init__(*args, **kwargs)
        set_field_attributes(self.visible_fields())


class AddActivityForm(forms.ModelForm):
    date = forms.DateTimeField(
        input_formats=[DATETIMEPICKER_FORMAT],
        widget=forms.DateTimeInput(attrs={"class": "form-control datetimepicker"}),
    )

    class Meta:
        model = Activity
        exclude = ("trace_file", "created", "modified", "is_demo_activity", "evaluates_for_awards")

    def __init__(self, *args, **kwargs):
        super(AddActivityForm, self).__init__(*args, **kwargs)
        self.fields["sport"] = forms.ModelChoiceField(queryset=Sport.objects.all().exclude(name="unknown"))
        set_field_attributes(self.visible_fields())


class EditActivityForm(forms.ModelForm):
    date = forms.DateTimeField(
        input_formats=[DATETIMEPICKER_FORMAT],
        widget=forms.DateTimeInput(attrs={"class": "form-control datetimepicker"}),
    )

    class Meta:
        model = Activity
        exclude = ("trace_file", "created", "modified", "is_demo_activity")

    def __init__(self, *args, **kwargs):
        super(EditActivityForm, self).__init__(*args, **kwargs)
        self.fields["sport"] = forms.ModelChoiceField(queryset=Sport.objects.all().exclude(name="unknown"))
        set_field_attributes(self.visible_fields())


class EditSettingsForm(forms.ModelForm):
    class Meta:
        model = Settings
        exclude = ("number_of_days", "created", "modified")

    def __init__(self, *args, **kwargs):
        super(EditSettingsForm, self).__init__(*args, **kwargs)
        set_field_attributes(self.visible_fields())



"""
ASGI entrypoint. Configures Django and then runs the application
defined in the ASGI_APPLICATION setting.
"""

import os

import django_eventstream
from channels.auth import AuthMiddlewareStack
from channels.routing import ProtocolTypeRouter, URLRouter
from django.conf.urls import url
from django.core.asgi import get_asgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "server.settings")

application = ProtocolTypeRouter(
    {
        "http": URLRouter(
            [
                url(
                    r"^events/",
                    AuthMiddlewareStack(URLRouter(django_eventstream.routing.urlpatterns)),
                    {"channels": ["event"]},
                ),
                url(r"", get_asgi_application()),
            ]
        ),
    }
)



from django.contrib import admin

from wkz import models

admin.site.register(models.Sport)
admin.site.register(models.Activity)
admin.site.register(models.Settings)
admin.site.register(models.Traces)
admin.site.register(models.Lap)
admin.site.register(models.BestSection)



import datetime
import logging
import os

import pytz
from django.contrib import messages
from django.forms import modelformset_factory
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.views.generic import DeleteView

from wkz import configuration as cfg
from wkz.awards_views import (
    get_ascent_ranking_of_activity,
    get_top_awards_for_one_sport,
)
from wkz.best_sections.generic import activity_suitable_for_awards
from wkz.file_helper.gpx_exporter import save_activity_to_gpx_file
from wkz.forms import DATETIMEPICKER_FORMAT, AddActivityForm, EditActivityForm
from wkz.models import Activity, BestSection, Lap, Sport
from wkz.plotting.plot_time_series import plot_time_series
from wkz.views import MapView, get_all_form_field_ids
from workoutizer import settings as django_settings

log = logging.getLogger(__name__)


class ActivityView(MapView):
    template_name = "activity/activity.html"

    def get(self, request, activity_id):
        activity = Activity.objects.get(id=activity_id)
        context = super(ActivityView, self).get(request=request, list_of_activities=[activity])
        settings = context["settings"]
        setattr(settings, "every_nth_value", cfg.every_nth_value)
        context["settings"] = settings
        activity_context = {
            "sports": Sport.objects.all().order_by("name"),
            "activity": activity,
            "form_field_ids": get_all_form_field_ids(),
            "fastest_sections": BestSection.objects.filter(activity=activity, kind="fastest"),
            "climb_sections": BestSection.objects.filter(activity=activity, kind="climb"),
            "page_name": activity.name,
            "is_activity_page": True,
        }
        if activity.trace_file:
            script_time_series, div_time_series, number_of_plots = plot_time_series(activity)
            activity_context["script_time_series"] = script_time_series
            activity_context["div_time_series"] = div_time_series
            activity_context["map_height"] = _get_map_height(number_of_plots)
        laps = Lap.objects.filter(trace=activity.trace_file, trigger="manual")
        if laps:
            activity_context["laps"] = laps
        activity_context["evaluates_for_awards"] = False
        if activity_suitable_for_awards(activity):
            activity_context["top_fastest_awards"] = get_top_awards_for_one_sport(
                sport=activity.sport, top_score=cfg.rank_limit, kinds=["fastest"]
            )
            activity_context["top_climb_awards"] = get_top_awards_for_one_sport(
                sport=activity.sport, top_score=cfg.rank_limit, kinds=["climb"]
            )
            activity_context["ascent_awards_ranking"] = get_ascent_ranking_of_activity(activity)
            activity_context["evaluates_for_awards"] = True
        return render(request, self.template_name, {**context, **activity_context})


def _get_map_height(number_of_plots: int) -> int:
    height = number_of_plots * 120 + 40
    if height < 400:
        return 400
    else:
        return height


def add_activity_view(request):
    sports = Sport.objects.all().order_by("name")
    if request.method == "POST":
        form = AddActivityForm(request.POST)
        if form.is_valid():
            instance = form.save()
            instance.save()
            messages.success(request, f"Successfully added '{form.cleaned_data['name']}'")
            return HttpResponseRedirect(reverse("home"))
        else:
            log.warning(f"form invalid: {form.errors}")
    else:
        form = AddActivityForm(initial={"date": str(datetime.datetime.now().strftime(DATETIMEPICKER_FORMAT))})
    return render(
        request,
        "activity/add_activity.html",
        {"sports": sports, "form": form, "form_field_ids": get_all_form_field_ids(), "page_name": "Add Activity"},
    )


def edit_activity_view(request, activity_id):
    form_field_ids = get_all_form_field_ids()
    sports = Sport.objects.all().order_by("name")
    activity = Activity.objects.get(id=activity_id)
    date = activity.date.astimezone(pytz.timezone(django_settings.TIME_ZONE))
    date = str(date.strftime(DATETIMEPICKER_FORMAT))
    activity_form = EditActivityForm(request.POST or None, instance=activity, initial={"date": date})
    laps = Lap.objects.filter(trace=activity.trace_file, trigger="manual")
    has_laps = True if laps else False
    if has_laps:
        LapFormSet = modelformset_factory(Lap, fields=("label",))
        formset = LapFormSet(request.POST or None, queryset=laps)
        form_field_ids = _add_formset_field_ids(form_field_ids, formset)
    else:
        formset = None
    if request.method == "POST":
        if activity_form.is_valid():
            log.debug(f"got valid activity_form: {activity_form.cleaned_data}")
            activity_form.save()
            if has_laps:
                if formset.is_valid():
                    formset.save()
            messages.success(request, f"Successfully modified '{activity_form.cleaned_data['name']}'")
            return HttpResponseRedirect(f"/activity/{activity_id}")
        else:
            log.warning(f"form invalid: {activity_form.errors}")
            if has_laps:
                log.warning(f"form invalid: {activity_form.errors}")
    return render(
        request,
        "activity/edit_activity.html",
        {
            "form": activity_form,
            "sports": sports,
            "activity": activity,
            "formset": formset,
            "has_laps": has_laps,
            "form_field_ids": form_field_ids,
            "page_name": f"Edit Activity: {activity.name}",
        },
    )


def _add_formset_field_ids(form_field_ids, formset):
    """
    Add all input fields of a given form set to the list of all field ids in order to avoid
    keyboard navigation while entering text in form fields.
    """

    for i, form in enumerate(formset):
        for field in form.base_fields.keys():
            field = f"id_form-{i}-{field}"
            form_field_ids.append(field)
    return form_field_ids


def download_activity(request, activity_id):
    activity = Activity.objects.get(id=activity_id)
    path = save_activity_to_gpx_file(activity=activity)
    if os.path.exists(path):
        with open(path, "rb") as fh:
            response = HttpResponse(fh.read(), content_type="application/vnd.ms-excel")
            response["Content-Disposition"] = "inline; filename=" + os.path.basename(path)
            return response
    raise Http404


class ActivityDeleteView(DeleteView):
    template_name = "activity/activity_confirm_delete.html"
    model = Activity
    slug_field = "activity_id"
    success_url = "/"

    def get(self, request, *args, **kwargs):
        sports = Sport.objects.all().order_by("name")
        activity = Activity.objects.get(id=kwargs["pk"])
        return render(
            request,
            self.template_name,
            {
                "sports": sports,
                "activity": activity,
                "form_field_ids": get_all_form_field_ids(),
                "page_name": f"Delete Activity: {activity.name}",
            },
        )


class DemoActivityDeleteView(DeleteView):
    template_name = "activity/demo_activity_confirm_delete.html"
    activities = Activity.objects.filter(is_demo_activity=True)

    def get(self, request, *args, **kwargs):
        sports = Sport.objects.all().order_by("name")
        log.debug(f"activities to be deleted: {self.activities}")
        return render(
            request,
            self.template_name,
            {
                "sports": sports,
                "activities": self.activities,
                "form_field_ids": get_all_form_field_ids(),
                "page_name": "Delete Demo Activities",
            },
        )

    def post(self, request, *args, **kwargs):
        log.debug(f"deleting: {self.activities}")
        for activity in self.activities:
            activity.delete()
        log.info("deleted demo activities")
        return HttpResponseRedirect(reverse("home"))



import os

import pytest

from wkz.file_helper.fit_parser import FITParser
from wkz.file_helper.gpx_exporter import gpx_header
from wkz.file_helper.gpx_parser import GPXParser


@pytest.fixture
def fit_parser(test_data_dir, demo_data_dir) -> FITParser:
    test_file_path = "example.fit"

    def _pass_path(path=test_file_path):
        file_in_test_data_dir = os.path.join(test_data_dir, path)
        file_in_demo_data_dir = os.path.join(demo_data_dir, path)
        if os.path.isfile(file_in_test_data_dir):
            return FITParser(path_to_file=file_in_test_data_dir, md5sum=None)
        elif os.path.isfile(file_in_demo_data_dir):
            return FITParser(path_to_file=file_in_demo_data_dir, md5sum=None)
        else:
            raise FileNotFoundError(f"file {path} neither found in {test_data_dir} nor in {demo_data_dir}")

    return _pass_path


@pytest.fixture
def gpx_parser(test_data_dir):
    test_file_path = os.path.join(test_data_dir, "example.gpx")

    def _pass_path(path=test_file_path):
        return GPXParser(path_to_file=path, md5sum="foo")

    return _pass_path


@pytest.fixture
def trace_coordinates():
    return [
        [8.476648433133962, 49.48468884453178],
        [8.476595375686886, 49.48457719758154],
        [8.47659705206752, 49.48453864082695],
        [8.47659654915333, 49.48450796306134],
    ]


@pytest.fixture
def trace_coordinates_with_elevation():
    return [
        [8.476648433133962, 49.48468884453178, 200],
        [8.476595375686886, 49.48457719758154, 201],
        [8.47659705206752, 49.48453864082695, 202],
        [8.47659654915333, 49.48450796306134, 203],
    ]


@pytest.fixture
def gpx_string():
    return f"""{gpx_header}
    <metadata>
        <time>2019-07-12T00:00:00Z</time>
        <link href="https://github.com/fgebhart/workoutizer">
            <text>Workoutizer</text>
        </link>
    </metadata>
    <trk>
        <name>test</name>
            <type>Running</type>
        <trkseg>
            <trkpt lat="49.48468884453178" lon="8.476648433133962">
                <time>2019-07-12T12:00:00Z</time>
            </trkpt>
            <trkpt lat="49.48457719758154" lon="8.476595375686886">
                <time>2019-07-12T12:01:00Z</time>
            </trkpt>
            <trkpt lat="49.48453864082695" lon="8.47659705206752">
                <time>2019-07-12T12:02:00Z</time>
            </trkpt>
            <trkpt lat="49.48450796306134" lon="8.47659654915333">
                <time>2019-07-12T12:03:00Z</time>
            </trkpt>
            
        </trkseg>
    </trk>
</gpx>
"""


@pytest.fixture
def gpx_string_with_elevation():
    return f"""{gpx_header}
    <metadata>
        <time>2019-07-12T00:00:00Z</time>
        <link href="https://github.com/fgebhart/workoutizer">
            <text>Workoutizer</text>
        </link>
    </metadata>
    <trk>
        <name>test</name>
            <type>Running</type>
        <trkseg>
            <trkpt lat="49.48468884453178" lon="8.476648433133962">
                <time>2019-07-12T12:00:00Z</time>
                <ele>200</ele>
            </trkpt>
            <trkpt lat="49.48457719758154" lon="8.476595375686886">
                <time>2019-07-12T12:01:00Z</time>
                <ele>201</ele>
            </trkpt>
            <trkpt lat="49.48453864082695" lon="8.47659705206752">
                <time>2019-07-12T12:02:00Z</time>
                <ele>202</ele>
            </trkpt>
            <trkpt lat="49.48450796306134" lon="8.47659654915333">
                <time>2019-07-12T12:03:00Z</time>
                <ele>203</ele>
            </trkpt>
            
        </trkseg>
    </trk>
</gpx>
"""



import datetime

from wkz.forms import AddActivityForm, AddSportsForm, EditActivityForm, EditSettingsForm


def test_edit_settings_form():
    form_data = {
        "path_to_trace_dir": "/home/pi/traces/",
        "path_to_garmin_device": "/home/pi/traces/",
        "number_of_days": 30,
        "delete_files_after_import": False,
    }
    form = EditSettingsForm(data=form_data)
    assert form.is_valid() is True


def test_add_sport_form(db):
    form_data = {
        "name": "Running",
        "color": "#060087",
        "icon": "running",
    }
    form = AddSportsForm(data=form_data)
    assert form.is_valid() is True


def test_add_and_edit_activity_form(db, sport):
    form_data = {
        "name": "Running",
        "sport": sport,
        "date": datetime.date.today(),
        "duration": datetime.timedelta(minutes=30),
        "distance": 5.2,
        "description": "some super sport",
    }
    form = AddActivityForm(data=form_data)
    assert form.is_valid() is True

    form = EditActivityForm(data=form_data)
    assert form.is_valid() is True



import logging

import gpxpy
import gpxpy.gpx

from wkz.file_helper.parser import Parser
from wkz.gis.geo import get_total_distance_of_trace

log = logging.getLogger(__name__)


class GPXParser(Parser):
    def __init__(self, path_to_file: str, md5sum: str):
        super(GPXParser, self).__init__(path_to_file, md5sum)
        self.gpx = None

    def _parse_metadata(self):
        gpx_file = open(self.path_to_file, "r")
        self.file_name = self.get_file_name_from_path(self.path_to_file)
        self.gpx = gpxpy.parse(gpx_file)
        self._get_sport_from_gpx_file()
        self._get_duration_from_gpx_file()

    def _get_sport_from_gpx_file(self):
        if self.gpx.tracks[0].type:
            self.sport = self.gpx.tracks[0].type
        else:
            for e in self.gpx.tracks[0].extensions:
                if e.tag.split("}")[1] == "activity":
                    self.sport = e.text
                    log.debug(f"found sport: {self.sport}")

    def _get_duration_from_gpx_file(self):
        all_points_time = []
        for s in self.gpx.tracks[0].segments:
            for p in s.points:
                all_points_time.append(p.time)
        start = all_points_time[0]
        end = all_points_time[-1]
        if start and end:
            self.duration = end - start
            log.debug(f"found duration: {self.duration}")
        else:
            log.warning("could not find duration")
        if self.gpx.time:
            self.date = self.gpx.time
            log.debug(f"found date: {self.date}")
        else:
            self.date = start
            log.debug(f"found date: {self.date}")

        if not self.date:
            log.warning("could not find date in GPX file, will use OS file created date")
            self.get_file_created_datetime()

    def _parse_records(self):
        for track in self.gpx.tracks:
            for segment in track.segments:
                for point in segment.points:
                    if point.elevation:
                        self.altitude_list.append(point.elevation)
                    self.latitude_list.append(point.latitude)
                    self.longitude_list.append(point.longitude)
                    if point.time:
                        self.timestamps_list.append(point.time.timestamp())
        log.debug(f"found number of coordinates: {len(self.longitude_list)}")
        log.debug(f"found number of timestamps: {len(self.timestamps_list)}")
        log.debug(f"found number of elevation points: {len(self.altitude_list)}")

    def _post_process_data(self):
        self.distance = get_total_distance_of_trace(
            longitude_list=self.longitude_list,
            latitude_list=self.latitude_list,
        )
        log.debug(f"found distance: {self.distance}")



from typing import Dict, List, Union

from django.shortcuts import render

from wkz import configuration as cfg
from wkz import models
from wkz.views import WKZView

awards_info_texts = {
    "general": (
        f"This page lists the top {cfg.rank_limit} activities in the respective category. Both individual "
        "activities and entire sports can be disabled for awards."
    ),
    "fastest": (
        "The fastest sections are determined by computing the average velocity over the given section distance."
    ),
    "climb": (
        "The best climb sections are determined by computing the accumulated evelation gain per minute over the "
        "given section distance."
    ),
    "ascent": ("The total ascent equals the aggregated sum of all uphill meters gained during one activity."),
}


class AwardsViews(WKZView):
    template_name = "awards/awards.html"

    def get(self, request):
        self.context["page_name"] = "Awards"

        # get fastest awards
        top_fastest_awards = get_top_awards_for_all_sports(top_score=cfg.rank_limit, kinds=["fastest"])
        self.context["top_fastest_awards"] = top_fastest_awards

        # get climb awards
        top_climb_awards = get_top_awards_for_all_sports(top_score=cfg.rank_limit, kinds=["climb"])
        self.context["top_climb_awards"] = top_climb_awards

        # get ascent awards
        top_ascent_awards = _get_top_ascent_awards_for_all_sports()
        self.context["top_ascent_awards"] = top_ascent_awards

        # get info text for hover over question mark
        self.context["info_text"] = awards_info_texts

        return render(request, template_name=self.template_name, context=self.context)


def _get_top_ascent_awards_for_all_sports() -> Dict[models.Sport, list]:
    top_awards = {}
    for sport in models.Sport.objects.filter(evaluates_for_awards=True).exclude(name="unknown").order_by("name"):
        top_awards[sport] = _get_top_ascent_awards_for_one_sport(sport)
    return top_awards


def get_ascent_ranking_of_activity(activity: models.Activity) -> Union[int, None]:
    list_of_ascent_awards = _get_top_ascent_awards_for_one_sport(activity.sport)
    if activity in list_of_ascent_awards:
        return list_of_ascent_awards.index(activity) + 1
    else:
        return None


def _get_top_ascent_awards_for_one_sport(sport: models.Sport) -> list:
    return list(
        (
            models.Activity.objects.filter(
                sport=sport,
                evaluates_for_awards=True,
            )
            .exclude(trace_file__total_ascent=None)
            .order_by("-trace_file__total_ascent")[: cfg.rank_limit]
        )
    )


def _get_best_sections_of_sport_and_distance(
    sport: models.Sport, distance: int, top_score: int, kinds: List[str]
) -> List[models.BestSection]:
    awards_per_distance = list(
        models.BestSection.objects.filter(
            activity__sport=sport,
            activity__evaluates_for_awards=True,
            distance=distance,
            kind__in=kinds,
        ).order_by("-max_value")[:top_score]
    )
    return awards_per_distance


def get_top_awards_for_one_sport(sport: models.Sport, top_score: int, kinds: List[str]) -> List[models.BestSection]:
    awards = []
    for bs in cfg.best_sections:
        if bs["kind"] in kinds:
            for distance in bs["distances"]:
                awards_per_distance = _get_best_sections_of_sport_and_distance(sport, distance, top_score, kinds)
                for rank, section in enumerate(awards_per_distance):
                    setattr(section, "rank", rank + 1)
                if awards_per_distance:
                    awards += awards_per_distance
    return awards


def get_top_awards_for_all_sports(top_score: int, kinds: List[str]) -> Dict[models.Sport, List[models.BestSection]]:
    top_awards = {}
    for sport in models.Sport.objects.filter(evaluates_for_awards=True).exclude(name="unknown").order_by("name"):
        awards = get_top_awards_for_one_sport(sport, top_score, kinds)
        if awards:
            top_awards[sport] = awards
    return top_awards



import logging
import os
import signal

import psutil
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

from wkz.file_helper.fit_collector import try_to_mount_device

log = logging.getLogger(__name__)


@api_view(["POST"])
def mount_device_endpoint(request):
    try:
        mount_path = try_to_mount_device()
        if mount_path:
            return Response("mounted and checked for files", status=status.HTTP_200_OK)
        else:
            log.error(f"could not mount device, no valid mount path available - got: {mount_path}")
            return Response("failed", status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    except Exception as e:
        log.error(f"could not mount device: {e}", exc_info=True)
        return Response("failed", status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["POST"])
def stop_django_server(request):
    log.info("Stopped.")
    pid = os.getpid()
    parent = psutil.Process(pid)

    # first kill all child processes
    for child in parent.children(recursive=True):
        os.kill(child.pid, signal.SIGINT)

    # lastely kill the parent process
    os.kill(pid, signal.SIGINT)
    return Response("stopped", status=status.HTTP_200_OK)



import logging
from dataclasses import dataclass
from math import acos, cos, pi, sin
from typing import List, Tuple

import pandas as pd
from geopy.geocoders import Nominatim
from geopy.point import Point

from wkz import configuration as cfg

log = logging.getLogger(__name__)


@dataclass
class GeoTrace:
    pk: int
    name: str
    coordinates: list
    sport: str
    color: str = "#808080"
    opacity: float = 0.7
    weight: float = 3.0


def _to_rad(degree: float) -> float:
    return degree / 180 * pi


def calculate_distance_between_points(coordinate_1: Tuple[float], coordinate_2: Tuple[float]) -> float:
    if coordinate_1[0] == coordinate_2[0] and coordinate_1[1] == coordinate_2[1]:
        return 0.0
    distance = acos(
        sin(_to_rad(coordinate_1[0])) * sin(_to_rad(coordinate_2[0]))
        + cos(_to_rad(coordinate_1[0])) * cos(_to_rad(coordinate_2[0])) * cos(_to_rad(coordinate_1[1] - coordinate_2[1]))
    )
    # multiply by earth radius (nominal "zero tide" equatorial) in centimeter
    return distance * 6378100


def get_total_distance_of_trace(longitude_list: List[float], latitude_list: List[float]) -> float:
    if len(latitude_list) != len(longitude_list):
        raise ValueError("lat and lon lists need to have same length")
    if len(latitude_list) < 2 or len(longitude_list) < 2:
        raise ValueError("lat and lon lists need to be at least of length 2")
    coordinates_df = pd.DataFrame({"lat": latitude_list, "lon": longitude_list})
    coordinates_df.dropna(inplace=True)

    total_distance = 0
    for index, row in coordinates_df.iterrows():
        if index < len(coordinates_df) - 1:
            point = (row["lat"], row["lon"])
            next_point = (coordinates_df.at[index + 1, "lat"], coordinates_df.at[index + 1, "lon"])
            dist = calculate_distance_between_points(point, next_point)
            total_distance += dist
    return round(total_distance / 1000, 2)


def add_elevation_data_to_coordinates(coordinates: list, altitude: list):
    coordinates_with_elevation = []
    for coordinate, altitude in zip(coordinates, altitude):
        coordinate = coordinate + (altitude,)
        coordinates_with_elevation.append(coordinate)
    return coordinates_with_elevation


def get_location_name(coordinate: Tuple[float, float]) -> str:
    app = Nominatim(user_agent="workoutizer")
    try:
        p = Point(coordinate[0], coordinate[1])
        address = app.reverse(query=p, language="en", timeout=10).raw["address"]
        # use name of location from village, town or city (in this order)
        if "village" in address.keys():
            return address["village"]
        elif "town" in address.keys():
            return address["town"]
        elif "city" in address.keys():
            return address["city"]
    except (TypeError, ValueError, AttributeError):
        return None


def get_list_of_coordinates(list_of_lon: List[float], list_of_lat: List[float]) -> List[Tuple[float]]:
    return list(
        zip(
            list(pd.Series(list_of_lon, dtype="float64").iloc[:: cfg.every_nth_value].ffill().bfill()),
            list(pd.Series(list_of_lat, dtype="float64").iloc[:: cfg.every_nth_value].ffill().bfill()),
        )
    )



# Generated by Django 3.1.6 on 2021-02-18 21:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wkz", "0002_auto_20210130_1351"),
    ]

    operations = [
        migrations.AlterField(
            model_name="settings",
            name="number_of_days",
            field=models.IntegerField(
                choices=[
                    (9999, "all"),
                    (1095, "last 3 years"),
                    (730, "last 2 years"),
                    (365, "last year"),
                    (180, "last 180 days"),
                    (90, "last 90 days"),
                    (30, "last 30 days"),
                    (10, "last 10 days"),
                ],
                default=30,
            ),
        ),
    ]
